package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.util.ArrayList;
import java.util.List;

import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.service.SysUserDutyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaMessage;
import com.example.civilizedcity.service.TaMessageService;

/**
 * 消息通知;(ta_message)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2023-1-2
 */
@Api(tags = "消息通知对象功能接口")
@RestController
@RequestMapping("/")
public class TaMessageController extends BaseController {

    @Autowired
    private TaMessageService taMessageService;

    @Autowired
    private SysUserDutyService sysUserDutyService;

    /**
     * 通过ID查询单条数据
     *
     * @param msgId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taMessage/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taMessageService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taMessage")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) throws Exception {

        SysUser sysUser = currentUser();
        List<String> dutyList = sysUserDutyService.getListByUser(sysUser.getUserId());
        if (null == dutyList) {
            dutyList = new ArrayList<>();
        }

        IPage<TaMessage> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaMessage> queryWrapper = new QueryWrapper<>();

        // 如果是管理员
        if (dutyList.contains(Constants.DUTY_MANAGER)) {
            // 可以查询当前单位以及给自己的
            queryWrapper.nested(q -> q.eq("org_id", Constants.ROOT_ORG).or().eq("receive_user", sysUser.getUserId()));
        } else if (dutyList.contains(Constants.DUTY_INSPECTOR)) {
            // 督察员只能查询给自己的
            queryWrapper.eq("receive_user", sysUser.getUserId());
        } else if (dutyList.contains(Constants.DUTY_ORG_USER)) {
            // 交办单位的，可以查询给本单位的
            queryWrapper.eq("org_id", sysUser.getOrgId());
            queryWrapper.ne("msg_type", "apply-verify");
        } else if (dutyList.contains(Constants.DUTY_ORG_MANAGER)) {
            // 交办单位管理员, 只能查询 问题单提审 类型的消息
            queryWrapper.eq("org_id", sysUser.getOrgId());
            queryWrapper.eq("msg_type", "apply-verify");
        }

        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");
        IPage<TaMessage> result = taMessageService.page(pg);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taMessage 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taMessage")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaMessage taMessage) throws Exception {
        taMessageService.save(taMessage);
        return ResponseBean.success(taMessage);
    }

    /**
     * 更新数据
     *
     * @param taMessage 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taMessage/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaMessage taMessage,
                             @ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        taMessageService.updateById(taMessage);
        return ResponseBean.success(taMessage);
    }

    /**
     * 通过主键删除数据
     *
     * @param msgId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taMessage/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id) {
        taMessageService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}