package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.mapper.TaAttachMapper;
import com.example.civilizedcity.mapper.TaIssueMapper;
import com.example.civilizedcity.mapper.TaPersonMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.TaFeedback;
import com.example.civilizedcity.mapper.TaFeedbackMapper;
import com.example.civilizedcity.service.TaFeedbackService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 反馈表;(ta_feedback)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2023-1-30
 */
@Service
public class TaFeedbackServiceImpl extends BaseServiceImpl<TaFeedbackMapper, TaFeedback> implements TaFeedbackService {

    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaAttachMapper taAttachMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNew(TaFeedback taFeedback, TaPerson taPerson) throws Exception {

        // 准备 TaFeedback 数据
        taFeedback.setFeedbackId(StringUtils.uuid());

        if (StringUtils.isEmpty(taFeedback.getName())) {
            taFeedback.setName(taPerson.getName());
        } else {
            if (StringUtils.isEmpty(taPerson.getName())) {
                taPerson.setName(taFeedback.getName());
                taPersonMapper.updateById(taPerson);
            }
        }

        if (StringUtils.isEmpty(taFeedback.getPhone())) {
            taFeedback.setPhone(taPerson.getPhone());
        }

        taFeedback.setPersonId(taPerson.getPersonId());
        taFeedback.setStatus(Constants.STATUS_NORMAL);
        taFeedback.setCreateDate(LocalDateTime.now());

        // 准备 TaIssue 数据
        TaIssue taIssue = new TaIssue();
        taIssue.setTitle(taFeedback.getTitle());
        taIssue.setLocation(taFeedback.getLocation());
        taIssue.setAddr(taFeedback.getAddr());
        taIssue.setContent(taFeedback.getContent());

        taIssue.setOrgId(null); // 暂时没有责任单位
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessStatus(Constants.PROCESS_STATUS_READY);
        taIssue.setProcessNum(1);
        taIssue.setSourceType(Constants.ISSUE_SOURCE_FEEDBACK);
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreateUser(taFeedback.getPersonId());
        taIssue.setUserName(taFeedback.getName());
        taIssue.setCreateDate(LocalDateTime.now());
        taIssueMapper.insert(taIssue);
        // 保存文件
        List<TaAttach> attachList = taFeedback.getAttachList();
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                item.setAttachId(null);
                item.setOwnerType(Constants.ATTACH_SOURCE_ISSUE);
                item.setOwnerId(taIssue.getIssueId().toString());
                item.setStatus(Constants.STATUS_NORMAL);
                item.setCreateDate(LocalDateTime.now());
                taAttachMapper.insert(item);
            }
        }

        taFeedback.setIssueId(taIssue.getIssueId().toString());
        save(taFeedback);
        // 保存文件
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                item.setAttachId(null);
                item.setOwnerType(Constants.ATTACH_SOURCE_FEEDBACK);
                item.setOwnerId(taFeedback.getFeedbackId());
                taAttachMapper.insert(item);
            }
        }

    }

    @Override
    public IPage<TaIssue> getPageBy(IPage<TaIssue> pg, String personId, String bizStatus) {

        List<String> processList = new ArrayList<>();
        String verifyStatus = null;

        // 未处理
        if (Constants.PROCESS_START.equals(bizStatus)) {
            processList.add(Constants.PROCESS_START);
        }

        // 已处理, 包含处理中以及已完成的
        if (Constants.PROCESS_ASSIGNED.equals(bizStatus)) {
            processList.add(Constants.PROCESS_ASSIGNED);
            processList.add(Constants.PROCESS_END);
        }

        // 已打回
        if (Constants.APPLY_STATUS_REJECT.equals(bizStatus)) {
            verifyStatus = Constants.APPLY_STATUS_REJECT;
        }

        return baseMapper.getPageBy(pg, personId, processList, verifyStatus);
    }
}