package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaIssueProcess;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.mapper.TaFeedbackMapper;
import com.example.civilizedcity.mapper.TaIssueProcessMapper;
import com.example.civilizedcity.mapper.TaOrgIssueMapper;
import com.example.civilizedcity.vo.StatVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.mapper.TaIssueMapper;
import com.example.civilizedcity.service.TaIssueService;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 问题表;(ta_issue)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class TaIssueServiceImpl extends BaseServiceImpl<TaIssueMapper, TaIssue> implements TaIssueService {

    @Autowired
    TaOrgIssueMapper taOrgIssueMapper;

    @Autowired
    TaFeedbackMapper taFeedbackMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Override
    public IPage<TaIssue> getPageBy(IPage<TaIssue> pg, String orgId) {
        if (null == orgId) {
            return baseMapper.getAllBy(pg);
        }


        return baseMapper.getPageBy(pg, orgId);
    }

    @Override
    public void updateValidateStatus(Integer issueId, int validateStatus) {
        baseMapper.updateValidateStatus(issueId, validateStatus);
    }

    @Override
    public void updateApply(Integer issueId, String applyType, Integer applyId) {
        baseMapper.updateApply(issueId, applyType, applyId);
    }

    @Override
    public List<StatVo> statMaIndex(SysUser user, TaPerson person, String duty) {
        List<StatVo> result = new ArrayList<>();

        // 督察员
        if (Constants.DUTY_INSPECTOR.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(user.getUserId());
            result.add(new StatVo().setName("已上报").setValue(getStatValueBy(res.get("published_num"), 0)));
            result.add(new StatVo().setName("已交办").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("未交办").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
        }

        // 管理员
        else if (Constants.DUTY_MANAGER.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(null);
            result.add(new StatVo().setName("待交办").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("已交办").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
        }

        // 单位人员 或者 单位管理员
        else if (Constants.DUTY_ORG_USER.equals(duty) || Constants.DUTY_ORG_MANAGER.equals(duty)) {
            Map<String, Object> res = taOrgIssueMapper.statMaIndex(user.getOrgId());
            result.add(new StatVo().setName("处理中").setValue(getStatValueBy(res.get("doing_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
            result.add(new StatVo().setName("已逾期").setValue(getStatValueBy(res.get("delay_num"), 0)));
        }

        // 查询人员
        else if (Constants.DUTY_QUERY_PERSON.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(null);
            result.add(new StatVo().setName("未处理").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("处理中").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
        }

        // 市民
        else {
            Map<String, Object> res = taFeedbackMapper.statMaIndex(person.getPersonId());
            result.add(new StatVo().setName("未处理").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("已处理").setValue(getStatValueBy(res.get("assigned_num"), 0) + getStatValueBy(res.get("end_num"), 0)));
            result.add(new StatVo().setName("已打回").setValue(getStatValueBy(res.get("reject_num"), 0)));
        }

        return result;
    }

    @Override
    public void updateEnd(SysUser sysUser, String orgId, Integer issueId) {
        // 问题单完成
        baseMapper.updateProcess(issueId, sysUser.getUserId(), Constants.PROCESS_END, Constants.PROCESS_END);

        // 机构问题单完成
        taOrgIssueMapper.updateProcess(issueId, orgId, Constants.PROCESS_END);

        // 流程完成
        TaIssueProcess taIssueProcess = new TaIssueProcess();
        taIssueProcess.setIssueId(issueId);
        taIssueProcess.setOrgId(orgId);
        taIssueProcess.setProcessNode(Constants.PROCESS_END);
        taIssueProcess.setProcessStatus(Constants.PROCESS_END);
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);

    }

    Integer getStatValueBy(Object v, Integer def) {
        if (null == v) return def;
        return Integer.valueOf(v.toString());
    }
}